require File.expand_path(File.dirname(__FILE__) + '/spec_helper')

describe "CtgbIdsConverter version" do
  subject { CtgbIdsConverter::VERSION }
  it {should be_a_kind_of(String)}
  it {should match(/\d+\.\d+\.\d+/)}
end

describe "CtgbIdsConverter configuration attributes for service on linux2" do
  before(:each) do
    CtgbIdsConverter.configure do |config|
      config.host = 'localhost'
      config.token = 'zU4vFmORUQ9DBJPlJlt'
    end
  end
  subject {CtgbIdsConverter.configuration}
  its(:hydra){ should be_a_kind_of(Typhoeus::Hydra)}
  its(:host){ should == 'localhost'}
  its(:token){ should == 'zU4vFmORUQ9DBJPlJlt'}
  its(:port){ should == 80}
  context "when set port to 3001" do
    before(:each) do
      CtgbIdsConverter.configuration.port = 3001
    end
    its(:port){ should == 3001}
  end
end

# following test are based on ids_converter web service running locally on port 3001 with two test user
describe CtgbIdsConverter::RemoteConverter do
  subject(:converter) { CtgbIdsConverter::RemoteConverter }
  before(:each) do
    CtgbIdsConverter.configure do |config|
      config.host = 'localhost'
      config.token = 'zU4vFmORUQ9DBJPlJlt'
      config.port = 3001
    end
  end

  describe ".connected?" do
    it "returns true" do
      converter.connected?.should be_true
    end
  end

  describe ".code" do
    context "when providing a correct public id" do
      it "returns coding" do
        converter.code('AN000127').should == 'XX999341'
      end
    end
    context "when providing a wrong public id" do
      specify { expect {converter.code('AN000127xs')}.to raise_error CtgbIdsConverter::WrongIdException }
      specify { expect {converter.code('')}.to raise_error CtgbIdsConverter::WrongIdException }
    end
    context "when providing a new public id but user has no intert privileges" do
      # for this to work AN000000 should be missing in the conversion id DB (ids_pair table)
      before(:each) do
        CtgbIdsConverter.configuration.token = 'dFdwK5Hhk5d3ocIebea' #test user without insert public id privilege
      end
      specify { expect {converter.code('AN000000')}.to raise_error CtgbIdsConverter::UnauthorizedConversionException }
    end
  end

  describe ".decode" do
    context "when providing a correct clinic id" do
      it "returns decoding" do
        converter.decode('XX999341').should == 'AN000127'
      end
    end
    context "when providing a wrong clinic id" do
      specify { expect {converter.decode('XX999341xxx')}.to raise_error CtgbIdsConverter::WrongIdException }
      specify { expect {converter.decode('')}.to raise_error CtgbIdsConverter::WrongIdException }
    end
  end
end
